unit wsapih; {WebSite Api header file}

interface
uses
  SysUtils,
  Classes,
  Windows;

const
  MAJOR_VERSION	= 1;
  MINOR_VERSION	= 1;

// ------
// Limits
// ------
MAX_TYPE_LEN	= 64; // Longest postproc content type
MAX_PROC_DLLS	= 32; // Max # postprocessors
MAX_NAME_LEN	= 64;				// Longest CGI DLL name
MAX_CGI_DLLS	= 32;				// MAX # CGI DLLs
SML_STRING_LEN 	= 64;				// Small string buffer
MED_STRING_LEN	= 256;				// Medium string buffer
MAX_STRING_LEN	=1024;			// Big string buffer
HUGE_STRING_LEN =8192;			// Very large string buffer
MAX_EXT_HEADERS =100;				// Max extra request or CGI/1.1 headers

//-------------------
//	HTTP Result Codes
//-------------------
DOCUMENT_FOLLOWS=200;
CREATED		=201;
ACCEPTED=202;
PARTIAL_INFORMATION	=203;
NO_RESPONSE		=204;
DELETED			=205;
MODIFIED		=206;
MOVED			=301;
REDIRECT		=302;
USE_LOCAL_COPY		=304;
BAD_REQUEST		=400;
AUTH_REQUIRED		=401;
FORBIDDEN		=403;
NOT_FOUND		=404;
SERVER_ERROR		=500;
NOT_IMPLEMENTED		=501;

//--------------------
//	Supported methods
//--------------------
//
// Numeric method codes used in some interfaces
//
M_GET		=0;
M_PUT		=1;
M_POST		=2;
M_DELETE	=3;
M_LINK		=4;
M_UNLINK	=5;
M_ANY		=255;
//---------------
//
// Returned by url_to_file()
//
REDIRECT_URL	=-1;
STD_DOCUMENT	=0;
SCRIPT_DOS	=1;		// Used as intfc parameter to exec_cgi_script
SCRIPT_WIN	=2;		// Used as intfc parameter to exec_cgi_script
SCRIPT_SHELL	=4;		// Used as intfc parameter to exec_cgi_script

// --------------------------------
// Authentication Hook Return Codes
// --------------------------------
AUTH_REJECT  =0;		// Reject username/password
AUTH_ACCEPT  =1;		// Accept username/password
AUTH_INTERNAL=2;		// Use internal authentication
//---------------------
//	Access option codes
//---------------------
//SECOPT_OR_CLASS_USERS	 // "OR" logic for class/user access
//SECOPT_SSL_ONLY	 // Accessible via SSL only
//SECOPT_NO_BROWSE	 // Disable directory browsing

Type
  {$A+}
  xhdr=Record
    key:PChar;
    value:PChar;
  end;

  PCtx=^TCtx;
  TCtx=record
    link:Pointer;                 // Don't even THINK about this one
    // Server Configuration
    _hMainWnd:HWND ;              // Server's Main window
    bService:BOOL;                // TRUE if running as NT service
    bTraceHook:BOOL ;             // TRUE if want hook tracing
    temp_path:PChar;              // Path to server's temp directory
    server_version:PChar;         // Server's version string for headers
    // Low-level
    hSocket:integer; {SOCKET;}        // Client socket descriptor
    dwPort:DWORD;                 // IP port for this request
    dwThreadID:DWORD;             // Current Thread ID
    dwStartTicks:DWORD;           // SystemTicks at thread start
    bSSL:BOOL;                    // TRUE = Running on SSL
    // Remote (client) identity
    remote_host:PChar;            // Client's host name or NULL if not found
    remote_ip:PChar;              // Clients IP address (dotted quad)
    remote_name:PChar;            // Client's host name if found, else IP address
    // Local (server) identity
    local_name:PChar;             // Identity for multi-home
    url_prefix:PChar;             // Multi-home URL prefix
    local_CN:PChar;               // Certificate Common Name (CN)
    cert:Pbyte;                   // DER-format Certificate
    cert_len:DWORD;               // Length of Certificate
    // HTTP Request info - may be changed by pre/post processor
    http_request:Array[0..MED_STRING_LEN-1] of char;  // HTTP raw request line (1st 256 chars)  of76
    http_09:BOOL;                                     // TRUE if HTTP/0.9 request
    method:Array[0..SML_STRING_LEN-1] of char;        // HTTP method  of336
    url:Array[0..MAX_STRING_LEN-1] of char;           // HTTP URL (decoded)  of400 ok
    params:Array[0..MAX_STRING_LEN-1] of char;        // HTTP URL Parameters (';' delimited)  of1424
    args:Array[0..MAX_STRING_LEN-1] of char;          // HTTP arguments      of1680
    http_accept:Array[0..MAX_STRING_LEN-1] of char;   // HTTP "Accept" line  of3472
    if_mod_since:Array[0..SML_STRING_LEN-1] of char;  // HTTP "If-Modified-Since"
    request_range:Array[0..MAX_STRING_LEN-1] of char; // HTTP "Request-Range"
    unl_mod_since:Array[0..SML_STRING_LEN-1] of char; // HTTP "Unless Modified Since"
    referer:Array[0..MAX_STRING_LEN-1] of char;       // HTTP "Referer"
    from:Array[0..MED_STRING_LEN-1] of char;          // HTTP "From"
    user_agent:Array[0..MED_STRING_LEN-1] of char;    // HTTP "User-Agent"
    keep_alive:BOOL;                                  // TRUE if request has Connection: Keep-Alive
    // Access Control & Authentication  (HTTP "Authorization")
    auth_line:Array[0..MED_STRING_LEN-1] of char;     // "Authorization" line
    auth_type:PChar;                           // "Authorization" type (strdup'd)
    auth_name:PChar;                           // "Authorization" Realm (strdup'd)
    user:Array[0..SML_STRING_LEN-1] of char;   // User name
    pass:Array[0..SML_STRING_LEN-1] of char;   // Password
    allow_options:DWORD;                       // Access options (SECOPT_ constants)
    // Request content (if any), response content
    content_length:LongInt;                    // HTTP "Content-Length"
    content_type:Array[0..MED_STRING_LEN-1] of char;      // HTTP "Content-Type"
    content_encoding:Array[0..MED_STRING_LEN-1] of char;  // HTTP "Content-Encoding"
    location:Array[0..MAX_STRING_LEN-1] of char;          // HTTP "Location"
    last_modified:Array[0..SML_STRING_LEN-1] of char;     // HTTP "Last-Modified"
    // Response status line
    rsp_status:Array[0..SML_STRING_LEN-1] of char; // e.g., HTTP/1.0 200 OK
    // Extra header tuples.
    num_req_xhdr:LongInt;                          // Number of request extra headers
    req_xhdr:Array[0..MAX_EXT_HEADERS-1] of xhdr;  // Extra headers in request.
    num_rsp_xhdr:LongInt;                          // Number of response extra headers
    rsp_xhdr:Array[0..MAX_EXT_HEADERS-1] of xhdr;  // Extra headers for response.
    // Authentication Info
    user_to_check:Array[0..SML_STRING_LEN-1] of char; // Server checks this username
    // Open file for response (postprocessing hooks only!)
    hOutFile:THandle;
    // Results (use only for logging!)
    http_status:integer;          // HTTP/1.0 status code of trans.
    bytes:LongInt;                // #bytes sent, excluding header
    // User contexts
    user_1:Dword;
    user_2:Dword;
  end;
  {$A-}

Procedure process_get(meth,url,args:PChar;var tp:TCtx);  cdecl;
Function  bind_wsapi(major,minor:word;strict:BOOL):BOOL; cdecl;
Procedure send_http_header(var tp:TCtx;do_xhdr:BOOL);    cdecl;
Procedure die(atype:integer;aString:PChar;var tp:TCtx);  cdecl;
Procedure nwrite(buf:PChar;nreq:integer;var tp:TCtx);    cdecl;
Procedure nflush(var tp:TCtx);                           cdecl;

implementation {----------------------------------------------}

Procedure process_get(meth,url,args:PChar;var tp:TCtx);  external 'httpd32.exe' name 'process_get';
Function  bind_wsapi(major,minor:word;strict:BOOL):BOOL; external 'httpd32.exe' name 'bind_wsapi';
Procedure send_http_header(var tp:TCtx;do_xhdr:BOOL);    external 'httpd32.exe' name 'send_http_header';
Procedure die(atype:integer;aString:PChar;var tp:TCtx);  external 'httpd32.exe' name 'die';
Procedure nwrite(buf:PChar;nreq:integer;var tp:TCtx);    external 'httpd32.exe' name 'nwrite';
Procedure nflush(var tp:TCtx);                           external 'httpd32.exe' name 'nflush';

end.
