unit WSUrlVar; {Variador de arquivos 96 Tecepe }
{ 22/8/96  Omar F Reis
Filtro de subst de URI. Procura URI em uma tabela e troca arquivo por
arq equivalente se achar.
p.e. tcp.gif pode ser substituido por TCP1.gif, tcpmov.gif ou tcpult.gif
As substituicoes sao especificadas em um HttpVar.ini, que tem o formato:

;====  httpvar.ini (lowercase please) =====
;Administrative URLs (Keep them secret!)
[LIST]
ReloadListURL=/_reloadlist
ShowListURL=/_showlist

;List of variable URLs
[URLs]
1=_tcp.gif
2=_is.gif

;Files for '_tcp.gif'
[_tcp.gif]
1=tcp1.gif
2=tcpmov.gif
3=tcpult.gif
...

;Files for '_is.gif'
[_is.gif]
1=is1.gif
2=is2.gif
...
}

interface
Uses Windows,Classes,SysUtils,IniFiles;

type
  Char100=Array[0..99] of char;

  {List of files for one URL}
  TOneURL=Class(TStringList)
    IxProximo:integer;
    Constructor Create; virtual;
    Function    NameOfNextFile:String;
    Procedure   ReportTo(SL:TStringList);
  end;

  {List of URLs. ChangeOriginalURL() returns new URL, if available}
  TVarURLList=Class(TStringList)
    IniFileName:String;
    CritSection:TRTLCriticalSection;
    Constructor Create(aIniName:String);
    Destructor  Destroy;  virtual;
    Function    ChangeOriginalURL(var aArq:String):Boolean;
    Procedure   SalvaIniFile;
    Procedure   FreeURLList;
    Procedure   LoadLista;
    Function    ReturnList:TStringList;
  end;

Function  CheckURL(URL,NewURL:PChar;var aSL:TStringList):boolean;

implementation {=====================================}

var
  URLList:TVarURLList;
  ReloadListURL:Char100;    {URL you should call to reload the list}
  ShowListURL:Char100;      {URL you should call see the list}

{Variable URLs must start with '/_'.}
Function CheckURL(URL,NewURL:PChar;var aSL:TStringList):boolean;
var S:String; L,M:integer;
begin
  Result:=FALSE;
  aSL:=nil;
  L:=StrLen(URL);
  if (L<2) or (URL[1]<>'_') then exit;   {URLs must start with '/_'}
  {Critical section protects multiples accesses to global URLList  }
  S:=StrPas(URL);                        {Convert to pascal str    }
  EnterCriticalSection(URLList.CritSection);
  try
    M:=0;
    if (StrIComp(URL,ReloadListURL)=0) then M:=1
      else if (StrIComp(URL,ShowListURL)=0) then M:=2;
    if M<>0 then {Administrative URL}
      begin
        if M=1 then
          begin
            URLList.FreeURLList;  {destroy old list}
            URLList.LoadLista;    {reload httpvar.ini}
          end;
        aSL:=URLList.ReturnList;  {returns the list}
        Result:=TRUE;
      end
      else if URLList.ChangeOriginalURL(S) then
        begin
          StrPCopy(NewURL,S);
          Result:=TRUE;
        end;
  finally
  LeaveCriticalSection(URLList.CritSection);
  end;
end;

{--------------------  TOneURL. -------------------------}
Constructor TOneURL.Create;
begin
  Inherited Create;
  Duplicates:=dupIgnore;
  Sorted:=TRUE;
  IxProximo:=0;
end;

Function TOneURL.NameOfNextFile:String; {returns '' if none}
begin
  if Count>0 then
    begin
      Result:=Strings[IxProximo];
      inc(IxProximo);
      if IxProximo>=Count then IxProximo:=0;
    end
    else Result:='';
end;

Procedure TOneURL.ReportTo(SL:TStringList);
var i:integer;
begin
  if Count>0 then for i:=0 to Count-1 do
    SL.Add(IntToStr(i+1)+':'+Strings[i]+'<br>');
end;

{-------------------  TVarURLList. ---------------------}
Constructor TVarURLList.Create(aIniName:String);
begin
  Inherited Create;
  Duplicates:=dupIgnore;
  Sorted:=TRUE;
  IniFileName:=aIniName;
  InitializeCriticalSection(CritSection);
  StrPCopy(ReloadListURL,'/_reloadlist');
  StrPCopy(ShowListURL  ,'/_showlist');
  LoadLista;
end;

Procedure TVarURLList.FreeURLList;
var i:integer; LAE:TOneURL;
begin
  for i:=0 to Count-1 do
    begin
      LAE:=TOneURL(Objects[i]);
      LAE.Free;        {desaloca}
    end;
end;

Destructor TVarURLList.Destroy;
begin
  FreeURLList;
  DeleteCriticalSection(CritSection);
  inherited Destroy;
end;

Function TVarURLList.ReturnList:TStringList;
var i:integer; LAE:TOneURL; SL:TStringList;
begin
  SL:=TStringList.Create;
  SL.Add('<html><body>');
  SL.Add('<h1>Http var list of URLs</h1>');
  for i:=0 to Count-1 do
    begin
      SL.Add('<hr>');
      SL.Add('URI:'+Strings[i]+'<br>');
      LAE:=TOneURL(Objects[i]);
      if Assigned(LAE) then LAE.ReportTo(SL);
    end;
  SL.Add('<hr>');
  SL.Add('Generated by Httpvar Copr 96 Tecepe (www.tecepe.com.br)<br>');
  SL.Add('</body></html>');
  Result:=SL;
end;

Procedure TVarURLList.LoadLista;
const MAXARQUIVOSEQ=100;
var aIniFile:TIniFile; i,j:integer; aArq,aArqEq:String[50]; LAE:TOneURL;
begin
  Clear;
  try
    aIniFile:=TIniFile.Create(IniFileName);
  except
    exit;  {//erro na abertura do .ini (e isto gera excessao?)}
  end;
  StrPCopy(ReloadListURL,aIniFile.ReadString('LIST','ReloadListURL','/_reloadlist'));
  StrPCopy(ShowListURL  ,aIniFile.ReadString('LIST','ShowListURL'  ,'/_showlist'));
  {Read the list}
  for i:=0 to MAXARQUIVOSEQ-1 do
    begin
      aArq:=aIniFile.ReadString('URLs',IntToStr(i+1),'');
      if aArq<>'' then
        begin
          LAE:=TOneURL.Create; {Cria lista}
          for j:=0 to MAXARQUIVOSEQ-1 do
            begin
              aArqEq:=aIniFile.ReadString(aArq,IntToStr(j+1),''); {pega arq equiv}
              if aArqEq<>'' then LAE.Add(aArqEq)  {Se tem insere}
                else break;                       {acabou, termina esta lista }
            end;
          if LAE.Count>0 then
            begin
              AddObject(aArq,LAE);
            end
            else LAE.Destroy;  {arq sem nenhum arq corresp (?)}
        end
        else break; {acabou}
    end;
  aIniFile.Free;
end;

//Se achar, retorna true e nome do arq equiv em aArq
Function TVarURLList.ChangeOriginalURL(var aArq:String):Boolean;
var LAE:TOneURL;  ix:integer; aArqEq:String[50];
begin
  Result:=FALSE;
  if (Count>0) and Find(aArq,ix) then
    begin
      LAE:=TOneURL(Objects[ix]);
      aArqEq:=LAE.NameOfNextFile;
      if aArqEq<>'' then
        begin
          aArq:=aArqEq;
          Result:=TRUE;
        end;
    end;
end;

Procedure TVarURLList.SalvaIniFile;
var i,j:integer; aIniFile:TIniFile; LAE:TOneURL;
begin
  aIniFile:=TIniFile.Create(IniFileName);
  for i:=0 to count-1 do
    aIniFile.WriteString('Arquivos',IntToStr(i+1),Strings[i]);
  for i:=0 to count-1 do
   begin
     LAE:=TOneURL(Objects[i]);
     for j:=0 to LAE.Count-1 do
       aIniFile.WriteString(Strings[i],IntToStr(j+1),LAE.Strings[j]);
   end;
  aIniFile.Free;
end;

{=========================================================}
Procedure InitURLList; {carrega ini}
begin
  URLList:=TVarURLList.Create('httpvar.ini');
end;

initialization
  InitURLList;
end.
